@@ -14,14 +14,14 @@
 package handler
 
 import (
+	"fmt"
 	"net/http"
 	"sync"
 	"time"
 
-	"github.com/julienschmidt/httprouter"
-	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/client_golang/prometheus/promhttp"
-	"github.com/prometheus/common/log"
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
+	"github.com/prometheus/common/route"
 
 	"github.com/prometheus/pushgateway/storage"
 )
@@ -29,26 +29,33 @@ import (
 // Delete returns a handler that accepts delete requests.
 //
 // The returned handler is already instrumented for Prometheus.
-func Delete(ms storage.MetricStore) func(http.ResponseWriter, *http.Request, httprouter.Params) {
-	var ps httprouter.Params
+func Delete(ms storage.MetricStore, jobBase64Encoded bool, logger log.Logger) func(http.ResponseWriter, *http.Request) {
 	var mtx sync.Mutex // Protects ps.
 
-	instrumentedHandler := promhttp.InstrumentHandlerCounter(
-		httpCnt.MustCurryWith(prometheus.Labels{"handler": "delete"}),
-		http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
-			job := ps.ByName("job")
-			labelsString := ps.ByName("labels")
+	instrumentedHandler := InstrumentWithCounter(
+		"delete",
+		http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			job := route.Param(r.Context(), "job")
+			if jobBase64Encoded {
+				var err error
+				if job, err = decodeBase64(job); err != nil {
+					http.Error(w, fmt.Sprintf("invalid base64 encoding in job name %q: %v", job, err), http.StatusBadRequest)
+					level.Debug(logger).Log("msg", "invalid base64 encoding in job name", "job", job, "err", err.Error())
+					return
+				}
+			}
+			labelsString := route.Param(r.Context(), "labels")
 			mtx.Unlock()
 
 			labels, err := splitLabels(labelsString)
 			if err != nil {
 				http.Error(w, err.Error(), http.StatusBadRequest)
-				log.Debugf("Failed to parse URL: %v, %v", labelsString, err.Error())
+				level.Debug(logger).Log("msg", "failed to parse URL", "url", labelsString, "err", err.Error())
 				return
 			}
 			if job == "" {
 				http.Error(w, "job name is required", http.StatusBadRequest)
-				log.Debug("job name is required")
+				level.Debug(logger).Log("msg", "job name is required")
 				return
 			}
 			labels["job"] = job
@@ -60,9 +67,8 @@ func Delete(ms storage.MetricStore) func(http.ResponseWriter, *http.Request, htt
 		}),
 	)
 
-	return func(w http.ResponseWriter, r *http.Request, params httprouter.Params) {
+	return func(w http.ResponseWriter, r *http.Request) {
 		mtx.Lock()
-		ps = params
 		instrumentedHandler.ServeHTTP(w, r)
 	}
 }
