@@ -15,6 +15,7 @@ package handler
 
 import (
 	"bytes"
+	"context"
 	"errors"
 	"fmt"
 	"net/http"
@@ -23,10 +24,11 @@ import (
 	"time"
 
 	"github.com/go-kit/kit/log"
-
+	//lint:ignore SA1019 Dependencies use the deprecated package, so we have to, too.
 	"github.com/golang/protobuf/proto"
-	"github.com/julienschmidt/httprouter"
 	"github.com/matttproud/golang_protobuf_extensions/pbutil"
+	"github.com/prometheus/common/route"
+
 	dto "github.com/prometheus/client_model/go"
 
 	"github.com/prometheus/pushgateway/storage"
@@ -76,6 +78,16 @@ func (m *MockMetricStore) Ready() error {
 	return nil
 }
 
+func ctxWithParams(params map[string]string, mainReq *http.Request) context.Context {
+	ctx := mainReq.Context()
+
+	for key, value := range params {
+		ctx = route.WithParam(ctx, key, value)
+	}
+
+	return ctx
+}
+
 func TestHealthyReady(t *testing.T) {
 	mms := MockMetricStore{}
 	req, err := http.NewRequest("GET", "http://example.org/", &bytes.Buffer{})
@@ -101,9 +113,10 @@ func TestHealthyReady(t *testing.T) {
 func TestPush(t *testing.T) {
 	mms := MockMetricStore{}
 	mmsWithErr := MockMetricStore{err: errors.New("testerror")}
-	handler := Push(&mms, false, false, logger)
-	handlerWithErr := Push(&mmsWithErr, false, false, logger)
-	handlerBase64 := Push(&mms, false, true, logger)
+	// false, true, false → no replace, check consistency, no base64 encoding.
+	handler := Push(&mms, false, true, false, logger)
+	handlerWithErr := Push(&mmsWithErr, false, true, false, logger)
+	handlerBase64 := Push(&mms, false, true, true, logger)
 	req, err := http.NewRequest("POST", "http://example.org/", &bytes.Buffer{})
 	if err != nil {
 		t.Fatal(err)
@@ -111,7 +124,7 @@ func TestPush(t *testing.T) {
 
 	// No job name.
 	w := httptest.NewRecorder()
-	handler(w, req, httprouter.Params{})
+	handler(w, req)
 	if expected, got := http.StatusBadRequest, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -122,7 +135,11 @@ func TestPush(t *testing.T) {
 	// With job name, but no instance name and no content.
 	mms.lastWriteRequest = storage.WriteRequest{}
 	w = httptest.NewRecorder()
-	handler(w, req, httprouter.Params{httprouter.Param{Key: "job", Value: "testjob"}})
+	params := map[string]string{
+		"job": "testjob",
+	}
+
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusOK, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -146,13 +163,12 @@ func TestPush(t *testing.T) {
 		t.Fatal(err)
 	}
 	w = httptest.NewRecorder()
-	handler(
-		w, req,
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-			httprouter.Param{Key: "instance", Value: "testinstance"},
-		},
-	)
+	params = map[string]string{
+		"job":      "testjob",
+		"instance": "testinstance",
+	}
+
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusBadRequest, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -170,13 +186,13 @@ func TestPush(t *testing.T) {
 		t.Fatal(err)
 	}
 	w = httptest.NewRecorder()
-	handler(
-		w, req,
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-			httprouter.Param{Key: "labels", Value: "/instance/testinstance"},
-		},
-	)
+
+	params = map[string]string{
+		"job":    "testjob",
+		"labels": "/instance/testinstance",
+	}
+
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusOK, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -206,13 +222,11 @@ func TestPush(t *testing.T) {
 		t.Fatal(err)
 	}
 	w = httptest.NewRecorder()
-	handlerWithErr(
-		w, req,
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-			httprouter.Param{Key: "labels", Value: "/instance/testinstance"},
-		},
-	)
+	params = map[string]string{
+		"job":    "testjob",
+		"labels": "/instance/testinstance",
+	}
+	handlerWithErr(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusBadRequest, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -243,13 +257,11 @@ func TestPush(t *testing.T) {
 		t.Fatal(err)
 	}
 	w = httptest.NewRecorder()
-	handlerBase64(
-		w, req,
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "dGVzdC9qb2I="},                         // job="test/job"
-			httprouter.Param{Key: "labels", Value: "/instance@base64/dGVzdGluc3RhbmNl"}, // instance="testinstance"
-		},
-	)
+	params = map[string]string{
+		"job":    "dGVzdC9qb2I=",                      // job="test/job"
+		"labels": "/instance@base64/dGVzdGluc3RhbmNl", // instance="testinstance"
+	}
+	handlerBase64(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusOK, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -281,12 +293,10 @@ func TestPush(t *testing.T) {
 		t.Fatal(err)
 	}
 	w = httptest.NewRecorder()
-	handler(
-		w, req,
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-		},
-	)
+	params = map[string]string{
+		"job": "testjob",
+	}
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusOK, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -317,13 +327,12 @@ func TestPush(t *testing.T) {
 		t.Fatal(err)
 	}
 	w = httptest.NewRecorder()
-	handler(
-		w, req,
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-			httprouter.Param{Key: "labels", Value: "/instance/testinstance"},
-		},
-	)
+	params = map[string]string{
+		"job":    "testjob",
+		"labels": "/instance/testinstance",
+	}
+
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	// Note that a real storage shourd reject pushes with timestamps. Here
 	// we only make sure it gets through. Rejection is tested in the storage
 	// package.
@@ -337,6 +346,7 @@ func TestPush(t *testing.T) {
 	if expected, got := int64(1000), mms.lastWriteRequest.MetricFamilies["b"].GetMetric()[0].GetTimestampMs(); expected != got {
 		t.Errorf("Wanted protobuf timestamp %v, got %v.", expected, got)
 	}
+
 	// With job name and instance name and protobuf content.
 	mms.lastWriteRequest = storage.WriteRequest{}
 	buf := &bytes.Buffer{}
@@ -378,13 +388,11 @@ func TestPush(t *testing.T) {
 	}
 	req.Header.Set("Content-Type", "application/vnd.google.protobuf; encoding=delimited; proto=io.prometheus.client.MetricFamily")
 	w = httptest.NewRecorder()
-	handler(
-		w, req,
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-			httprouter.Param{Key: "labels", Value: "/instance/testinstance"},
-		},
-	)
+	params = map[string]string{
+		"job":    "testjob",
+		"labels": "/instance/testinstance",
+	}
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusOK, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -411,14 +419,14 @@ func TestDelete(t *testing.T) {
 	mms := MockMetricStore{}
 	handler := Delete(&mms, false, logger)
 	handlerBase64 := Delete(&mms, true, logger)
+	req := &http.Request{}
+	var params map[string]string
 
 	// No job name.
 	mms.lastWriteRequest = storage.WriteRequest{}
 	w := httptest.NewRecorder()
-	handler(
-		w, &http.Request{},
-		httprouter.Params{},
-	)
+	params = map[string]string{}
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusBadRequest, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -429,12 +437,12 @@ func TestDelete(t *testing.T) {
 	// With job name, but no instance name.
 	mms.lastWriteRequest = storage.WriteRequest{}
 	w = httptest.NewRecorder()
-	handler(
-		w, &http.Request{},
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-		},
-	)
+
+	params = map[string]string{
+		"job": "testjob",
+	}
+
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusAccepted, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -451,13 +459,13 @@ func TestDelete(t *testing.T) {
 	// With job name and instance name.
 	mms.lastWriteRequest = storage.WriteRequest{}
 	w = httptest.NewRecorder()
-	handler(
-		w, &http.Request{},
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "testjob"},
-			httprouter.Param{Key: "labels", Value: "/instance/testinstance"},
-		},
-	)
+
+	params = map[string]string{
+		"job":    "testjob",
+		"labels": "/instance/testinstance",
+	}
+
+	handler(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusAccepted, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
@@ -474,13 +482,13 @@ func TestDelete(t *testing.T) {
 	// With base64-encoded job name and instance name.
 	mms.lastWriteRequest = storage.WriteRequest{}
 	w = httptest.NewRecorder()
-	handlerBase64(
-		w, &http.Request{},
-		httprouter.Params{
-			httprouter.Param{Key: "job", Value: "dGVzdC9qb2I="},                         // job="test/job"
-			httprouter.Param{Key: "labels", Value: "/instance@base64/dGVzdGluc3RhbmNl"}, // instance="testinstance"
-		},
-	)
+
+	params = map[string]string{
+		"job":    "dGVzdC9qb2I=",
+		"labels": "/instance@base64/dGVzdGluc3RhbmNl",
+	}
+
+	handlerBase64(w, req.WithContext(ctxWithParams(params, req)))
 	if expected, got := http.StatusAccepted, w.Code; expected != got {
 		t.Errorf("Wanted status code %v, got %v.", expected, got)
 	}
