@@ -25,12 +25,11 @@ import (
 
 	"github.com/go-kit/kit/log"
 	"github.com/go-kit/kit/log/level"
-	"github.com/julienschmidt/httprouter"
 	"github.com/matttproud/golang_protobuf_extensions/pbutil"
-	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
 	"github.com/prometheus/common/expfmt"
 	"github.com/prometheus/common/model"
+	"github.com/prometheus/common/route"
 
 	dto "github.com/prometheus/client_model/go"
 
@@ -45,26 +44,30 @@ const (
 
 // Push returns an http.Handler which accepts samples over HTTP and stores them
 // in the MetricStore. If replace is true, all metrics for the job and instance
-// given by the request are deleted before new ones are stored.
+// given by the request are deleted before new ones are stored. If check is
+// true, the pushed metrics are immediately checked for consistency (with
+// existing metrics and themselves), and an inconsistent push is rejected with
+// http.StatusBadRequest.
 //
 // The returned handler is already instrumented for Prometheus.
 func Push(
-	ms storage.MetricStore, replace bool, jobBase64Encoded bool, logger log.Logger,
-) func(http.ResponseWriter, *http.Request, httprouter.Params) {
-	var ps httprouter.Params
+	ms storage.MetricStore,
+	replace, check, jobBase64Encoded bool,
+	logger log.Logger,
+) func(http.ResponseWriter, *http.Request) {
 	var mtx sync.Mutex // Protects ps.
 
 	handler := http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-		job := ps.ByName("job")
+		job := route.Param(r.Context(), "job")
 		if jobBase64Encoded {
 			var err error
 			if job, err = decodeBase64(job); err != nil {
-				http.Error(w, fmt.Sprintf("invalid base64 encoding in job name %q: %v", ps.ByName("job"), err), http.StatusBadRequest)
-				level.Debug(logger).Log("msg", "invalid base64 encoding in job name", "job", ps.ByName("job"), "err", err.Error())
+				http.Error(w, fmt.Sprintf("invalid base64 encoding in job name %q: %v", job, err), http.StatusBadRequest)
+				level.Debug(logger).Log("msg", "invalid base64 encoding in job name", "job", job, "err", err.Error())
 				return
 			}
 		}
-		labelsString := ps.ByName("labels")
+		labelsString := route.Param(r.Context(), "labels")
 		mtx.Unlock()
 
 		labels, err := splitLabels(labelsString)
@@ -105,10 +108,20 @@ func Push(
 		}
 		if err != nil {
 			http.Error(w, err.Error(), http.StatusBadRequest)
-			level.Debug(logger).Log("msg", "failed to parse text", "err", err.Error())
+			level.Debug(logger).Log("msg", "failed to parse text", "source", r.RemoteAddr, "err", err.Error())
 			return
 		}
 		now := time.Now()
+		if !check {
+			ms.SubmitWriteRequest(storage.WriteRequest{
+				Labels:         labels,
+				Timestamp:      now,
+				MetricFamilies: metricFamilies,
+				Replace:        replace,
+			})
+			w.WriteHeader(http.StatusAccepted)
+			return
+		}
 		errCh := make(chan error, 1)
 		errReceived := false
 		ms.SubmitWriteRequest(storage.WriteRequest{
@@ -142,14 +155,11 @@ func Push(
 
 	instrumentedHandler := promhttp.InstrumentHandlerRequestSize(
 		httpPushSize, promhttp.InstrumentHandlerDuration(
-			httpPushDuration, promhttp.InstrumentHandlerCounter(
-				httpCnt.MustCurryWith(prometheus.Labels{"handler": "push"}),
-				handler,
-			)))
+			httpPushDuration, InstrumentWithCounter("push", handler),
+		))
 
-	return func(w http.ResponseWriter, r *http.Request, params httprouter.Params) {
+	return func(w http.ResponseWriter, r *http.Request) {
 		mtx.Lock()
-		ps = params
 		instrumentedHandler.ServeHTTP(w, r)
 	}
 }
