@@ -25,8 +25,6 @@ import (
 
 	"github.com/go-kit/kit/log"
 	"github.com/go-kit/kit/log/level"
-	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/client_golang/prometheus/promhttp"
 
 	"github.com/prometheus/common/version"
 	"github.com/prometheus/pushgateway/storage"
@@ -61,8 +59,8 @@ func Status(
 	logger log.Logger,
 ) http.Handler {
 	birth := time.Now()
-	return promhttp.InstrumentHandlerCounter(
-		httpCnt.MustCurryWith(prometheus.Labels{"handler": "status"}),
+	return InstrumentWithCounter(
+		"status",
 		http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 			t := template.New("status")
 			t.Funcs(template.FuncMap{
